package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.dao.TdImagesMapper;
import com.community.huiju.dao.TpTransactionMapper;
import com.community.huiju.dao.TransactionLikeMapper;
import com.community.huiju.dao.TransactionReportMapper;
import com.community.huiju.model.*;
import com.community.huiju.service.ITransactionService;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.xml.crypto.Data;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 物业web端二手交易表 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2018-12-29
 */
@Service
public class TransactionServiceImpl extends ServiceImpl<TpTransactionMapper, TpTransaction> implements ITransactionService {
	
	@Autowired
	private TpTransactionMapper tpTransactionMapper;
	
	@Autowired
	private TdImagesMapper tdImagesMapper;

	@Autowired
	private TransactionLikeMapper transactionLikeMapper;

	@Autowired
	private TransactionReportMapper transactionReportMapper;


	@Override
	public ResponseBean getTransactionList(String parameter,Integer communityId) {
		ResponseBean responseBean = new ResponseBean();
		JSONObject object= JSONObject.parseObject(parameter);
		String transactionTitle = object.getString("transactionTitle");
		String userName = object.getString("userName");
		Integer id = object.getInteger("id");
		Integer status = object.getInteger("status");
		Integer pageNum = object.getInteger("pageNum");
		Integer pageSize = object.getInteger("pageSize");
		Page<TpTicket> page = new Page<>();
		page.setSize(pageSize == null ? 10 : pageSize);
		page.setCurrent(pageNum == null ? 1 : pageNum);
		// 分页查询
		List<TpTransaction> list = tpTransactionMapper.selectListByParams(page,transactionTitle,userName,id,status,communityId);

		list = list.stream().map( e-> {

			QueryWrapper<TransactionLike> transactionLike= new QueryWrapper<>();
			transactionLike.eq("transaction_id",e.getId());
			transactionLike.eq("like_status",2);
			//不喜欢的总数
			 Integer transactionLikeCount= transactionLikeMapper.selectCount(transactionLike);
			e.setTransactionNoLikeSum(transactionLikeCount);

			QueryWrapper<TransactionReport> TransactionReport= new QueryWrapper<>();
			TransactionReport.eq("transaction_id",e.getId());
			Integer TransactionReportCount= transactionReportMapper.selectCount(TransactionReport);
			e.setReportSum(TransactionReportCount);

			return e;
		}).collect(Collectors.toList());
		Map<String, Object> map = Maps.newHashMap();
		map.put("list",list);
		map.put("total",page.getTotal());
		map.put("pageNum", page.getCurrent());
		map.put("pageSize", page.getSize());
		responseBean.addSuccess(map);
		return responseBean;
	}
	
	/**
	 * 获取交易信息详情
	 *
	 * @param id
	 * @return
	 */
	@Override
	public ResponseBean getTransactionInfo(Integer id) {
		ResponseBean responseBean = new ResponseBean();
		TpTransaction tpTransaction = tpTransactionMapper.selectById(id);
		QueryWrapper<TdImages> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("uuid", id);
		queryWrapper.eq("type", "transaction");
		List<TdImages> imagesList = tdImagesMapper.selectList(queryWrapper);
		// 转换, 只保留 imageUrl 字段·
		List<String> stringList = imagesList.stream().map(e->new String(e.getImageUrl())).collect(Collectors.toList());
		Map<String,Object> map = Maps.newHashMap();
		map.put("tpTransaction",tpTransaction);
		map.put("list",stringList);
		responseBean.addSuccess(map);
		return responseBean;
	}
	
	/**
	 * 作废二手交易
	 *
	 * @param id
	 * @return
	 */
	@Override
	public ResponseBean updateTransactionById(Integer id,Integer userId) {
		ResponseBean responseBean = new ResponseBean();
		TpTransaction tpTransaction = new TpTransaction();
		tpTransaction.setId(id);
		tpTransaction.setStatus("0");
		tpTransaction.setUpdateUser(userId);
		tpTransaction.setUpdateDate(LocalDateTime.now());
		int count = tpTransactionMapper.updateById(tpTransaction);
		if (count > 0){
			responseBean.addSuccess(count);
			return responseBean;
		}
		responseBean.addError("作废出错");
		return responseBean;
	}

	@Override
	public ResponseBean transactionsAdd(String paramets, Integer communityId, Integer id) {
		ResponseBean response = new ResponseBean();
		JSONObject jsonobject = JSONObject.parseObject(paramets);
		TpTransaction tpTransaction = JSONObject.parseObject(paramets, TpTransaction.class);
		tpTransaction.setCommunityId(communityId);
		tpTransaction.setTaUserId(null);
		tpTransaction.setCreateDate(LocalDateTime.now());
		tpTransaction.setCreateUser(id);
		tpTransaction.setViewCount(0);
		JSONArray contentImg = jsonobject.getJSONArray("contentImg");
		String[] contentImgArray = contentImg.toArray(new String[]{});
		if (tpTransaction.getTransactionTitle() == null) {
			response.addError("标题不能为空");
			return response;
		}
		if (null == tpTransaction.getTransactionContent() && null == contentImg) {
			response.addError("内容和图片必须填一个");
			return response;
		}
		for (String img : contentImgArray) {
			TdImages tdImages = new TdImages();
			tdImages.setImageUrl(img);
			tdImages.setType("transaction");
			tdImages.setUuid(tpTransaction.getId());
			tdImages.setCreateTime(LocalDateTime.now());
			tdImages.setCreateUser(id);
			tdImagesMapper.insert(tdImages);
		}
		return response;

	}
}
